import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.NotYetConnectedException;

import com.sun.grizzly.ConnectorHandler;
import com.sun.grizzly.Controller;
import com.sun.grizzly.UDPConnectorHandler;
import com.sun.grizzly.Controller.Protocol;
import com.sun.grizzly.connectioncache.client.CacheableConnectorHandler;
import com.sun.grizzly.util.WorkerThread;

public class ConnectionManager{

	private static ConnectionManager INSTANCE;
	private Controller udpClientController = null;
	private InetAddress localAddress;
	
    private ConnectionManager() {
    	try{
    		this.localAddress = InetAddress.getLocalHost();
    	}catch(Exception e){}
   	}

    public static synchronized ConnectionManager getInstance(){
    	if(INSTANCE == null){
    		INSTANCE = new ConnectionManager();
    	}
    	return INSTANCE;
    }

    public void send(byte msg[], int length, String remote, int remotePort, int sendPort) throws IOException {
		InetSocketAddress local = new InetSocketAddress(localAddress, sendPort);
   		ConnectorHandler handler = createHandlerUDP(remote, remotePort, local, sendPort);
        
        ByteBuffer bb = toBuffer(msg, length);
        boolean isConnected = ((UDPConnectorHandler)(((CacheableConnectorHandler)handler).getUnderlyingConnectorHandler())).isConnected();
        boolean cacheBuffer = true;
        try {
        	cacheBuffer = write(handler, bb, isConnected, msg, length);
        } catch (NotYetConnectedException ex) {
        	throw new IOException("Not yet connected");
        } catch (IOException ex) {
        	throw ex;
        }finally{
        	if (bb != null && cacheBuffer) {
        		bb.clear();
        	}
            handler.close();
            udpClientController.releaseConnectorHandler(handler);
        }
    }

    private boolean write(ConnectorHandler handler, ByteBuffer bb, boolean isConnected, byte[] msg, int length) throws IOException{
        boolean cacheBuffer = true;
    	if (isConnected) {
    		long nWrite = handler.write(bb, false);
    		if (nWrite == 0) {
    			ByteBuffer copy = copy(msg, length);
    			if (handler instanceof CacheableConnectorHandler){
    				((UDPConnectorHandler)((CacheableConnectorHandler)handler).getUnderlyingConnectorHandler()).writeToAsyncQueue(copy);
    			}
    		}
    	} else {
    		cacheBuffer = false;
			ByteBuffer copy = copy(msg, length);
			if (handler instanceof CacheableConnectorHandler){
				((UDPConnectorHandler)((CacheableConnectorHandler)handler).getUnderlyingConnectorHandler()).writeToAsyncQueue(copy);
			}
    	}
    	
    	return cacheBuffer;
    }
    
    private ByteBuffer copy(byte[] msg, int length){
		ByteBuffer copy = ByteBuffer.allocate(length);
		copy.put(msg, 0, length);
		copy.flip();
		return copy;
    }
    
   	private final ByteBuffer toBuffer(byte[] msg, int length) throws UnsupportedEncodingException {
   		ByteBuffer bb = ((WorkerThread)Thread.currentThread()).getByteBuffer();
   		bb.clear();
        bb.put(msg, 0, length);
        bb.flip();
        return bb;
    }

    private ConnectorHandler createHandlerUDP(String remote, int remotePort, InetSocketAddress local, int sendPort) throws IOException {

    	ConnectorHandler connectorHandler = udpClientController.acquireConnectorHandler(Protocol.UDP);
    	InetSocketAddress socketRemote = new InetSocketAddress(remote, remotePort);

    	CustomUDPCallbackHandler callbackHandler 
    	= new CustomUDPCallbackHandler(connectorHandler, udpClientController, socketRemote);
        try {
        	connectorHandler.connect(socketRemote, local, callbackHandler);
            return connectorHandler;
        } catch (IOException e) {
        	e.printStackTrace();
            throw e;
        }
    }
    
    void setUDPClientController(Controller udpClientController){
    	this.udpClientController = udpClientController;        	
    }

}
