import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SelectionKey;

import com.sun.grizzly.CallbackHandler;
import com.sun.grizzly.ConnectorHandler;
import com.sun.grizzly.Context;
import com.sun.grizzly.Controller;
import com.sun.grizzly.IOEvent;
import com.sun.grizzly.UDPConnectorHandler;
import com.sun.grizzly.Controller.Protocol;
import com.sun.grizzly.connectioncache.client.CacheableConnectorHandler;

public class CustomUDPCallbackHandler implements CallbackHandler<Context>{
	private ConnectorHandler connectorHandler;
	private Controller udpClientController;
	
	public CustomUDPCallbackHandler(ConnectorHandler connectorHandler, Controller udpClientController, InetSocketAddress socketRemote){
		this.connectorHandler = connectorHandler;
		this.udpClientController = udpClientController;
	}
	
    public void onConnect(IOEvent<Context> ioEvent) {
        SelectionKey key = ioEvent.attachment().getSelectionKey();
        try {
            connectorHandler.finishConnect(key);
            if (((UDPConnectorHandler)((CacheableConnectorHandler)connectorHandler).getUnderlyingConnectorHandler()).isConnected()){
                udpClientController.registerKey(key, SelectionKey.OP_READ, Controller.Protocol.UDP);
            } else {
            	throw new IOException("Connect failed");
            }
        } catch (IOException ex) {
        	ex.printStackTrace();
            udpClientController.getSelectorHandler(Protocol.UDP).getSelectionKeyHandler().cancel(key);
        }
    }

    public void onRead(IOEvent<Context> ioEvent) {
        try {
            Context ctx = ioEvent.attachment();
            SelectionKey key = ctx.getSelectionKey();
            if (!key.isValid()) {
                return;
            }
            key.interestOps(key.interestOps() & (~SelectionKey.OP_READ));
            ctx.getProtocolChain().execute(ioEvent.attachment());
        } catch (Throwable e) {
        	e.printStackTrace();
        }
    }

    public void onWrite(IOEvent<Context> ioEvent) {
        Context ctx = ioEvent.attachment();
        SelectionKey key = ctx.getSelectionKey();
        key.interestOps(key.interestOps() & (~SelectionKey.OP_WRITE));
    }
}
