import java.net.InetAddress;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.CountDownLatch;
import com.sun.grizzly.Controller;
import com.sun.grizzly.ControllerStateListener;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultSelectionKeyHandler;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.ProtocolChainInstanceHandler;
import com.sun.grizzly.UDPSelectorHandler;
import com.sun.grizzly.connectioncache.client.CacheableConnectorHandlerPool;
import com.sun.grizzly.filter.ReadFilter;
import com.sun.grizzly.ControllerStateListenerAdapter;
import com.sun.grizzly.util.DefaultThreadPool;

public class MessageProcessor implements Runnable {

	public static DefaultSelectionKeyHandler keyHandler = new DefaultSelectionKeyHandler();
	public DefaultThreadPool serverThreadPool = null;
	public DefaultThreadPool clientThreadPool = null;
	public ConnectionManager connectionManager = null;
	public ProtocolChain protocolChain = null;
	public ProtocolChainInstanceHandler instanceHandler = null;
	protected Controller serverController = new Controller();
	protected Controller clientController = new Controller();
	protected Thread  thread;
	
	public void run(){
        final CountDownLatch serverLatch = new CountDownLatch(1);
        serverController.addStateListener(new ControllerStateListenerAdapter() {
            @Override public void onReady() {
                serverLatch.countDown();
            }

            @Override public void onException(Throwable e) {
            	e.printStackTrace();
                if (serverLatch.getCount() > 0) {
                    serverLatch.countDown();
                }
            }
        });
        new Thread(serverController).start();
        
        try{
        	serverLatch.await(30, TimeUnit.SECONDS);
        }catch(InterruptedException e){
        	e.printStackTrace();
        }
        
        
        final CountDownLatch clientLatch = new CountDownLatch(1);
        clientController.addStateListener(new ControllerStateListenerAdapter() {
            @Override public void onReady() {
                clientLatch.countDown();
            }

            @Override public void onException(Throwable e) {
            	e.printStackTrace();
                if (clientLatch.getCount() > 0) {
                    clientLatch.countDown();
                }
            }
        });
        new Thread(clientController).start();
        
        try{
        	clientLatch.await(30, TimeUnit.SECONDS);
        }catch(InterruptedException e){
        	e.printStackTrace();
        }
	}

    protected void initPipeline() {
    	serverThreadPool = new DefaultThreadPool();
    	clientThreadPool = serverThreadPool;
    }

	public synchronized void start() {
		if(this.protocolChain == null){
			this.protocolChain = getProtocolChain();
			this.instanceHandler = new SimpleProtocolChainInstanceHandler(protocolChain);
		}
		initPipeline();
		startServer();
		startClient();
	
		connectionManager = ConnectionManager.getInstance();
		connectionManager.setUDPClientController(clientController);	
		
		this.thread = new Thread(this);
		thread.setDaemon(false);
		thread.setName("UDPMessageProcessorThread");
		thread.start();
	}
    
	protected ProtocolChain getProtocolChain() {
		DefaultProtocolChain protocolChain = new DefaultProtocolChain();
		protocolChain.setContinuousExecution(false);

		ReadFilter readFilter = new ReadFilter();
		readFilter.setContinuousExecution(false);

		protocolChain.addFilter(readFilter);
		protocolChain.addFilter(new MessageFilter());

		return protocolChain;
	}

	protected void startServer(){
		UDPSelectorHandler udpSelector = createUdpEndpoint();
		serverController.addSelectorHandler(udpSelector);
		udpSelector.setSelectionKeyHandler(MessageProcessor.keyHandler);
		serverController.setProtocolChainInstanceHandler(this.instanceHandler);
		serverController.setThreadPool(serverThreadPool);
		serverController.setReadThreadsCount(0);

		class ReadyListener implements ControllerStateListener {
			public void onStarted(){}
			public void onReady(){
				try{
					UDPSelectorHandler udpServerSelectorHandler = (UDPSelectorHandler)serverController.getSelectorHandler(Controller.Protocol.UDP);
					udpServerSelectorHandler.setSocketNativeReceiveBufferSize(8192);
				}catch(Exception e){e.printStackTrace();}
			}
			public void onStopped(){}
			public void onException(Throwable e){
				e.printStackTrace();
			}
		}
		serverController.addStateListener(new ReadyListener());
//		serverController.setAllowContextCaching(true);
	}
	
	private UDPSelectorHandler createUdpEndpoint() {
		try{
			UDPSelectorHandler selectorHandler = new UDPSelectorHandler();
			selectorHandler.setPort(Constant.serverPort);
			selectorHandler.setInet(InetAddress.getLocalHost());
			selectorHandler.setReuseAddress(true);
			return selectorHandler;
		}catch(Exception e){e.printStackTrace();}
		return null;
	}
	
	protected void startClient(){
		clientController.setProtocolChainInstanceHandler(this.instanceHandler);
		clientController.setThreadPool(clientThreadPool);
		UDPSelectorHandler udpClientSelectorHandler = new UDPSelectorHandler(true);
		udpClientSelectorHandler.setSelectionKeyHandler(MessageProcessor.keyHandler);
		
		CacheableConnectorHandlerPool connectorHandlerPool = new CacheableConnectorHandlerPool(clientController, 100, 10, 1);
		clientController.setConnectorHandlerPool(connectorHandlerPool);
		
		clientController.addSelectorHandler(udpClientSelectorHandler);
		clientController.setReadThreadsCount(0);
//		clientController.setAllowContextCaching(true);
	}
	
	public static void main(String[] args){
		System.out.println("Server starting.....");
		MessageProcessor processor = new MessageProcessor();
		processor.start();
		System.out.println("Server started.");
		
		System.out.println("UAS starting.....");
		UAS uas = new UAS();
		uas.start();
		System.out.println("UAS started.");
		
		System.out.println("UAC starting.....");
		UAC uac = new UAC();
		uac.start();
		System.out.println("UAC started.");
	}
}
